/*
  This file is part of match-the-color
  Copyright (C) 2014, 2015 Stefano Verzegnassi

    This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License 3 as published by
  the Free Software Foundation.

    This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
  along with this program. If not, see http://www.gnu.org/licenses/.
*/

import QtQuick 2.0
import QtQuick.LocalStorage 2.0

ListModel {
    id: model

    property string databaseName
    property int maxNumberOfEntries: 10
    property real topScore: 0

    signal scoresCleared

    // *** functions
    function addScore(score) {
        internal.addEntry({ player: "Unknown", score: score, date: new Date() })
        internal.refreshModel()
    }

    function addPlayerScore(name, score) {
        internal.addEntry({ player: name, score: score, date: new Date() })
        internal.refreshModel()
    }

    function clearScores() {
        internal.db.transaction(function(tx) {
            tx.executeSql('DROP TABLE ' + model.databaseName)
        });

        model.clear();
        internal.createTable();
        internal.getTopScore();

        model.scoresCleared()
    }

    function getBestScoreOfTheDayFromDiffToday(diff) {
        var dayToCheck = new Date();
        dayToCheck.setDate(dayToCheck.getDate() - diff);
        var results;

        internal.db.transaction(function(tx) {
            results = tx.executeSql('SELECT * FROM ' + model.databaseName + ' ORDER BY score DESC')
        });

        for (var i = 0; i < results.rows.length; i++) {
            var entry = results.rows.item(i);

            if (new Date(entry.date).getDate() === dayToCheck.getDate())
                return entry;
        }

        return {
            "player": "Unknown",
            "score": 0,
            "date": dayToCheck
        }
    }

    Component.onCompleted: {
        internal.openDb();
        internal.createTable();
        internal.refreshModel();
    }

    property QtObject internal: QtObject {
        property var db;

        function openDb()
        {
            internal.db = LocalStorage.openDatabaseSync(model.databaseName, "1.0", model.databaseName, 1000000);
        }

        function createTable()
        {
            internal.db.transaction(function(tx) {
                tx.executeSql('CREATE TABLE IF NOT EXISTS ' + model.databaseName + ' ( id INTEGER PRIMARY KEY, player TEXT, score INTEGER, date INTEGER )')
            });
        }

        function fillModel()
        {
            model.clear();

            internal.db.transaction(function(tx) {
                var results = tx.executeSql('SELECT * FROM ' + model.databaseName + ' ORDER BY score DESC')

                var numberOfEntriesToFill = Math.min(maxNumberOfEntries, results.rows.length)
                for (var i = 0; i < numberOfEntriesToFill; i++) {
                    var entry = results.rows.item(i);
                    model.append({
                        "id": parseInt(entry.id),
                        "player": entry.player,
                        "score": entry.score,
                        "date": entry.date
                    });
                }
            });
        }


        function refreshModel() {
            model.clear()
            internal.fillModel()
            internal.getTopScore()
        }

        function addEntry(entry)
        {
            internal.db.transaction(function(tx) {
                tx.executeSql('INSERT INTO ' + model.databaseName + ' (player, score, date) VALUES (?, ?, ?)',
                              [ entry.player, entry.score, entry.date ]);
            });
        }

        function getTopScore() {
            if (model.count > 0) {
                model.topScore = model.get(0).score
            } else {
                model.topScore = 0
            }
        }
    }
}
