/*
  This file is part of match-the-color
  Copyright (C) 2014, 2015 Stefano Verzegnassi

    This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License 3 as published by
  the Free Software Foundation.

    This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
  along with this program. If not, see http://www.gnu.org/licenses/.
*/

import QtQuick 2.4
import Ubuntu.Components 1.2
import QtQuick.Layouts 1.1
import Ubuntu.Components.Popups 1.0
import QtQml.Models 2.1

import "../components"
import "../components/qchart"
import "../components/qchart/QChart.js" as Charts

Page {
    id: highscorePage

    title: i18n.tr("High scores")
    head.sections.model: [ i18n.tr("Classic mode"), i18n.tr("Time attack") ]
    head.actions: Action {
        id: clearScore
        iconName: "reset"
        text: i18n.tr("Clear scores")
        onTriggered: PopupUtils.open(clearScoresDialog)
    }

    ListView {
        anchors.fill: parent
        clip: true

        header: topScoreHeader
        model: highscorePage.head.sections.selectedIndex == 0 ? classicHighScore : timeAttackHighScore
        delegate: ListItemBase {
            visible: model.index != 0   // We already show the top score in the view header

            Row {
                anchors {
                    fill: parent
                    margins: units.gu(0.5)
                }

                Label {
                    fontSize: "x-large"
                    text: "%1°".arg(model.index + 1)
                    width: parent.width - scoreCircle.width
                    anchors.verticalCenter: parent.verticalCenter
                }

                LogoItem {
                    id: scoreCircle
                    width: units.gu(5)
                    height: width
                    anchors.verticalCenter: parent.verticalCenter
                    rotationEnabled: false

                    Label {
                        anchors.centerIn: parent
                        text: model.score
                    }
                }
            }
        }
    }

    // *** COMPONENTS
    Component {
        id: topScoreHeader

        Item {
            height: units.gu(18)
            width: parent.width
            ListView {
                id: headerView
                anchors {
                    left: parent.left
                    right: parent.right
                }
                height: units.gu(18)
                orientation: ListView.Horizontal
                snapMode: ListView.SnapOneItem

                onContentXChanged: {
                    if (contentX < headerView.width)
                        currentIndex = 0
                    else
                        currentIndex = 1
                }

                Connections {
                    target: highscorePage.head.sections

                    onSelectedIndexChanged: {
                        headerView.positionViewAtBeginning()
                    }
                }

                model: ObjectModel {
                    ListItemBase {
                        height: units.gu(18)
                        width: headerView.width

                        RowLayout {
                            anchors.fill: parent

                            Item {
                                width: parent.height
                                height: width

                                LogoItem {
                                    anchors {
                                        fill: parent
                                        margins: units.gu(2)
                                    }

                                    Label {
                                        anchors.centerIn: parent
                                        fontSize: "x-large"
                                        text: highscorePage.head.sections.selectedIndex == 0 ? classicHighScore.topScore : timeAttackHighScore.topScore
                                    }
                                }
                            }

                            Label {
                                fontSize: "x-large"
                                text: i18n.tr("Top score")
                            }
                        }
                    }
                    ListItemBase {
                        height: units.gu(18)
                        width: headerView.width

                        Column {
                            anchors {
                                fill: parent
                                margins: units.gu(1)
                            }
                            Label { text: i18n.tr("Best scores this week") }

                            QChart {
                                id: scoreChart
                                width: parent.width
                                height: units.gu(12)

                                function getDayNameFromDiffToday(diff) {
                                    var day = new Date();
                                    day.setDate(day.getDate() - diff);

                                    return Qt.formatDate(day, "ddd")
                                }

                                function setChartValues() {
                                    scoreChart.chartOptions = {
                                        scaleOverlay: false,
                                        scaleOverride: false,
                                        scaleSteps: null,
                                        scaleStepWidth: null,
                                        scaleStartValue: null,
                                        scaleLineColor: "rgba(0,0,0,.1)",
                                        scaleLineWidth: 1,
                                        scaleShowLabels: true,
                                        scaleLabel: "<%=value%>",
                                        scaleFontFamily: "'Ubuntu'",
                                        scaleFontSize: FontUtils.sizeToPixels("small"),
                                        scaleFontStyle: "normal",
                                        scaleFontColor: "#666",
                                        scaleShowGridLines: true,
                                        scaleGridLineColor: "rgba(0,0,0,.05)",
                                        scaleGridLineWidth: 1,
                                        bezierCurve: true,
                                        pointDot: true,
                                        pointDotRadius: 4,
                                        pointDotStrokeWidth: 2,
                                        datasetStroke: true,
                                        datasetStrokeWidth: 2,
                                        datasetFill: true,
                                        animation: true,
                                        animationSteps: 60,
                                        animationEasing: "easeOutQuart",
                                        onAnimationComplete: null
                                    }

                                    var daysNameModel = [
                                                getDayNameFromDiffToday(6),
                                                getDayNameFromDiffToday(5),
                                                getDayNameFromDiffToday(4),
                                                getDayNameFromDiffToday(3),
                                                getDayNameFromDiffToday(2),
                                                getDayNameFromDiffToday(1),
                                                getDayNameFromDiffToday(0),
                                            ]


                                    var scoresModel;

                                    if (highscorePage.head.sections.selectedIndex == 0) {
                                        // Get results for classic mode
                                        scoresModel = [
                                                    classicHighScore.getBestScoreOfTheDayFromDiffToday(6).score,
                                                    classicHighScore.getBestScoreOfTheDayFromDiffToday(5).score,
                                                    classicHighScore.getBestScoreOfTheDayFromDiffToday(4).score,
                                                    classicHighScore.getBestScoreOfTheDayFromDiffToday(3).score,
                                                    classicHighScore.getBestScoreOfTheDayFromDiffToday(2).score,
                                                    classicHighScore.getBestScoreOfTheDayFromDiffToday(1).score,
                                                    classicHighScore.getBestScoreOfTheDayFromDiffToday(0).score,
                                                ]
                                    } else {
                                        scoresModel = [
                                                    timeAttackHighScore.getBestScoreOfTheDayFromDiffToday(6).score,
                                                    timeAttackHighScore.getBestScoreOfTheDayFromDiffToday(5).score,
                                                    timeAttackHighScore.getBestScoreOfTheDayFromDiffToday(4).score,
                                                    timeAttackHighScore.getBestScoreOfTheDayFromDiffToday(3).score,
                                                    timeAttackHighScore.getBestScoreOfTheDayFromDiffToday(2).score,
                                                    timeAttackHighScore.getBestScoreOfTheDayFromDiffToday(1).score,
                                                    timeAttackHighScore.getBestScoreOfTheDayFromDiffToday(0).score,
                                                ]
                                    }

                                    scoreChart.chartData = {
                                        labels: daysNameModel,
                                        datasets: [{
                                                fillColor: "rgba(220,220,220,0.5)",
                                                strokeColor: "rgba(220,220,220,1)",
                                                pointColor: "rgba(220,220,220,1)",
                                                pointStrokeColor: "#ffffff",
                                                data: scoresModel
                                            }]
                                    }

                                    // Force to reset chart, by setting it to null.
                                    // Then we can workaround the check on repainting
                                    scoreChart.chart = null
                                    scoreChart.repaint()
                                }

                                chartAnimated: true
                                chartAnimationEasing: Easing.InOutElastic
                                chartAnimationDuration: 2000
                                chartType: Charts.ChartType.LINE

                                Component.onCompleted: setChartValues()

                                Connections {
                                    target: highscorePage.head.sections
                                    onSelectedIndexChanged: scoreChart.setChartValues()
                                }

                                Connections {
                                    target: classicHighScore
                                    onScoresCleared: scoreChart.setChartValues()
                                }

                                Connections {
                                    target: timeAttackHighScore
                                    onScoresCleared: scoreChart.setChartValues()
                                }
                            }
                        }
                    }
                }
            }

            Row {
                anchors {
                    left: parent.left
                    right: parent.right
                    rightMargin: units.gu(2)
                    bottom: headerView.bottom
                    bottomMargin: units.gu(1)
                }
                height: units.gu(1)
                spacing: units.gu(0.25)
                layoutDirection: Qt.RightToLeft
                Repeater {
                    model: 2
                    delegate: Rectangle {
                        height: parent.height
                        width: height
                        radius: width * 0.5
                        color: model.index == headerView.currentIndex ? "#88888888" : "transparent"
                        border {
                            width: units.gu(0.25)
                            color: "#88888888"
                        }
                    }
                }
            }
        }
    }

    Component {
        id: clearScoresDialog

        Dialog {
            id: clearScoresDialogue

            // TRANSLATORS: This is the title of a dialog
            title: highscorePage.head.sections.selectedIndex == 0 ? i18n.tr("Clear classic mode scores")
                                                                  : i18n.tr("Clear time attack mode scores")
            text: i18n.tr("Scores are going to be deleted.\nAre you sure?")

            Button {
                text: i18n.tr("Cancel")
                onClicked: PopupUtils.close(clearScoresDialogue)
            }
            Button {
                text: i18n.tr("Delete")
                color: UbuntuColors.red
                onClicked: {
                    if (highscorePage.head.sections.selectedIndex == 0) {
                        classicHighScore.clearScores()
                    } else {
                        timeAttackHighScore.clearScores()
                    }

                    PopupUtils.close(clearScoresDialogue)
                }
            }
        }
    }
}
